package msgpack.rpcx.handler

import java.io. { PrintWriter, StringWriter }
import java.util.UUID

import javax.security.auth.login.LoginContext

import net.liftweb.common.Logger

import org.apache.activemq.jaas.JassCredentialCallbackHandler

import org.joda.time.LocalDateTime

import org.msgpack.rpc.Request

import msgpack.rpcx.auth. {

  AuthMap,
  CredentialException,
  LoginException,
  LoginInfo,
  LoginMap

}

/** Handles authentication and authorization.
 *
 * @author myst3r10n
 */
trait AuthHandler {

  import scala.collection.JavaConversions._

  /** List of authorizations. */
  protected val auths: AuthMap

  /** Log. */
  protected def log: Logger

  /** JAAS entry. */
  val jaas: String

  /** List of all logged users. */
  val loginMap: LoginMap

  /** Receives heartbeat from client.
   *
   * @param request of client
   * @param authId unique authentication id of client
   */
  def heartbeat(request: Request, authId: String) {

    try {

      isLoggedIn(authId)
      hasAuthorization(authId, "heartbeat")

      updateTimestamp(authId)

      request.sendResult(true)

    } catch {

      case e: Throwable =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        e.printStackTrace(printer)
        log.error(message.toString)
        request.sendError(e.toString)

    }
  }

  /** Receives login request from client.
   *
   * @param request of client
   * @param authId unique authentication id of client
   */
  def login(request: Request, username: String, password: String) {

    try {

      val context = new LoginContext(
        jaas,
        new JassCredentialCallbackHandler(username, password))

      context.login

      val authId = UUID.randomUUID.toString
      loginMap += authId -> new LoginInfo(context, username, new LocalDateTime)

      log.info("Logged in: " + username + " (" + authId + ")")

      request.sendResult(authId)

    } catch {

      case e: Throwable =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        e.printStackTrace(printer)
        log.error(message.toString)
        request.sendError(e.toString)

    }
  }

  /** Receives logout request from client.
   *
   * @param request of client
   * @param authId unique authentication id of client
   */
  def logout(request: Request, authId: String) {

    try {

      isLoggedIn(authId)

      loginMap(authId).context.logout

      val username = loginMap(authId).username

      loginMap -= authId

      log.info("Logged out: " + username + " (" + authId + ")")
      request.sendResult(true)

    } catch {

      case e: Throwable =>
        val message = new StringWriter
        val printer = new PrintWriter(message)
        e.printStackTrace(printer)
        log.error(message.toString)
        request.sendError(e.toString)


    }
  }

  /** Checks whether client has authorization.
   *
   * @param authId unique authentication id of client
   * @param function name where client need authorization
   */
  protected def hasAuthorization(
    authId: String,
    function: String) {

    if(auths.contains(function))
      asScalaSet(loginMap(authId).context
        .getSubject.getPrincipals).foreach(principal =>
        if(auths(function).contains(principal.getName))
          return)

    throw CredentialException(function, authId)

  }

  /** Checks whether client is logged in.
   *
   * #param authId unique authentication id of client
   */
  protected def isLoggedIn(authId: String) {

    if(!loginMap.contains(authId))
      throw LoginException(authId)

  }

  /** Updates timestamp of client.
   *
   * @param authId unique authentication id of client
   */
  protected def updateTimestamp(authId: String) {

    loginMap(authId).timestamp = new LocalDateTime

  }
}

